@@ -15,8 +15,13 @@
  *
  */
 
+#include <algorithm>
+#include <sstream>
+#include <string>
+
 #include "sdf/Assert.hh"
 #include "sdf/Element.hh"
+#include "sdf/Filesystem.hh"
 
 using namespace sdf;
 
@@ -31,8 +36,6 @@ Element::Element()
 /////////////////////////////////////////////////
 Element::~Element()
 {
-  delete this->dataPtr;
-  this->dataPtr = NULL;
 }
 
 /////////////////////////////////////////////////
@@ -45,6 +48,19 @@ ElementPtr Element::GetParent() const
 void Element::SetParent(const ElementPtr _parent)
 {
   this->dataPtr->parent = _parent;
+
+  // If this element doesn't have a path, get it from the parent
+  if (nullptr != _parent && (this->FilePath().empty() ||
+      this->FilePath() == "data-string"))
+  {
+    this->SetFilePath(_parent->FilePath());
+  }
+
+  // If this element doesn't have an original version, get it from the parent
+  if (nullptr != _parent && this->OriginalVersion().empty())
+  {
+    this->SetOriginalVersion(_parent->OriginalVersion());
+  }
 }
 
 /////////////////////////////////////////////////
@@ -97,26 +113,31 @@ std::string Element::ReferenceSDF() const
 
 /////////////////////////////////////////////////
 void Element::AddValue(const std::string &_type,
-    const std::string &_defaultValue, bool _required,
-    const std::string &_description)
+                       const std::string &_defaultValue,
+                       bool _required,
+                       const std::string &_description)
 {
   this->dataPtr->value = this->CreateParam(this->dataPtr->name,
       _type, _defaultValue, _required, _description);
 }
 
 /////////////////////////////////////////////////
-boost::shared_ptr<Param> Element::CreateParam(const std::string &_key,
-    const std::string &_type, const std::string &_defaultValue, bool _required,
-    const std::string &_description)
+ParamPtr Element::CreateParam(const std::string &_key,
+                              const std::string &_type,
+                              const std::string &_defaultValue,
+                              bool _required,
+                              const std::string &_description)
 {
-  return boost::shared_ptr<Param>(
-        new Param(_key, _type, _defaultValue, _required, _description));
+  return ParamPtr(
+      new Param(_key, _type, _defaultValue, _required, _description));
 }
 
 /////////////////////////////////////////////////
-void Element::AddAttribute(const std::string &_key, const std::string &_type,
-    const std::string &_defaultValue, bool _required,
-    const std::string &_description)
+void Element::AddAttribute(const std::string &_key,
+                           const std::string &_type,
+                           const std::string &_defaultValue,
+                           bool _required,
+                           const std::string &_description)
 {
   this->dataPtr->attributes.push_back(
       this->CreateParam(_key, _type, _defaultValue, _required, _description));
@@ -129,10 +150,11 @@ ElementPtr Element::Clone() const
   clone->dataPtr->description = this->dataPtr->description;
   clone->dataPtr->name = this->dataPtr->name;
   clone->dataPtr->required = this->dataPtr->required;
-  // clone->parent = this->dataPtr->parent;
   clone->dataPtr->copyChildren = this->dataPtr->copyChildren;
   clone->dataPtr->includeFilename = this->dataPtr->includeFilename;
   clone->dataPtr->referenceSDF = this->dataPtr->referenceSDF;
+  clone->dataPtr->path = this->dataPtr->path;
+  clone->dataPtr->originalVersion = this->dataPtr->originalVersion;
 
   Param_V::const_iterator aiter;
   for (aiter = this->dataPtr->attributes.begin();
@@ -152,11 +174,13 @@ ElementPtr Element::Clone() const
        eiter != this->dataPtr->elements.end(); ++eiter)
   {
     clone->dataPtr->elements.push_back((*eiter)->Clone());
-    clone->dataPtr->elements.back()->dataPtr->parent = clone;
+    clone->dataPtr->elements.back()->SetParent(clone);
   }
 
   if (this->dataPtr->value)
+  {
     clone->dataPtr->value = this->dataPtr->value->Clone();
+  }
 
   return clone;
 }
@@ -170,12 +194,16 @@ void Element::Copy(const ElementPtr _elem)
   this->dataPtr->copyChildren = _elem->GetCopyChildren();
   this->dataPtr->includeFilename = _elem->dataPtr->includeFilename;
   this->dataPtr->referenceSDF = _elem->ReferenceSDF();
+  this->dataPtr->originalVersion = _elem->OriginalVersion();
+  this->dataPtr->path = _elem->FilePath();
 
   for (Param_V::iterator iter = _elem->dataPtr->attributes.begin();
        iter != _elem->dataPtr->attributes.end(); ++iter)
   {
     if (!this->HasAttribute((*iter)->GetKey()))
+    {
       this->dataPtr->attributes.push_back((*iter)->Clone());
+    }
     ParamPtr param = this->GetAttribute((*iter)->GetKey());
     (*param) = (**iter);
   }
@@ -183,9 +211,13 @@ void Element::Copy(const ElementPtr _elem)
   if (_elem->GetValue())
   {
     if (!this->dataPtr->value)
+    {
       this->dataPtr->value = _elem->GetValue()->Clone();
+    }
     else
+    {
       *(this->dataPtr->value) = *(_elem->GetValue());
+    }
   }
 
   this->dataPtr->elementDescriptions.clear();
@@ -202,19 +234,30 @@ void Element::Copy(const ElementPtr _elem)
   {
     ElementPtr elem = (*iter)->Clone();
     elem->Copy(*iter);
-    elem->dataPtr->parent = shared_from_this();
+    elem->SetParent(shared_from_this());
     this->dataPtr->elements.push_back(elem);
   }
 }
 
 /////////////////////////////////////////////////
-void Element::PrintDescription(const std::string &_prefix)
+void Element::PrintDescription(const std::string &_prefix) const
 {
   std::cout << _prefix << "<element name ='" << this->dataPtr->name
-            << "' required ='" << this->dataPtr->required << "'>\n";
+            << "' required ='" << this->dataPtr->required << "'";
 
-  std::cout << _prefix << "  <description>" << this->dataPtr->description
-            << "</description>\n";
+  if (this->dataPtr->value)
+  {
+    std::cout << " type ='" << this->dataPtr->value->GetTypeName()
+              << "'"
+              << " default ='" << this->dataPtr->value->GetDefaultAsString()
+              << "'";
+  }
+
+  std::cout << ">\n";
+
+  std::cout << _prefix << "  <description><![CDATA["
+            << this->dataPtr->description
+            << "]]></description>\n";
 
   Param_V::iterator aiter;
   for (aiter = this->dataPtr->attributes.begin();
@@ -224,19 +267,23 @@ void Element::PrintDescription(const std::string &_prefix)
               << (*aiter)->GetKey() << "' type ='" << (*aiter)->GetTypeName()
               << "' default ='" << (*aiter)->GetDefaultAsString()
               << "' required ='" << (*aiter)->GetRequired() << "'>\n";
-    std::cout << _prefix << "    <description>" << (*aiter)->GetDescription()
-              << "</description>\n";
+    std::cout << _prefix << "    <description><![CDATA["
+              << (*aiter)->GetDescription()
+              << "]]></description>\n";
     std::cout << _prefix << "  </attribute>\n";
   }
 
   if (this->GetCopyChildren())
+  {
     std::cout << _prefix << "  <element copy_data ='true' required ='*'/>\n";
+  }
+
 
   std::string refSDF = this->ReferenceSDF();
   if (!refSDF.empty())
   {
-    std::cout << _prefix << "  <element ref ='" << refSDF <<
-        "' required ='*'/>\n";
+    std::cout << _prefix << "  <element ref ='" << refSDF
+              << "' required ='*'/>\n";
   }
 
   ElementPtr_V::iterator eiter;
@@ -250,7 +297,8 @@ void Element::PrintDescription(const std::string &_prefix)
 }
 
 /////////////////////////////////////////////////
-void Element::PrintDocRightPane(std::string &_html, int _spacing, int &_index)
+void Element::PrintDocRightPane(std::string &_html, int _spacing,
+                                int &_index) const
 {
   std::ostringstream stream;
   ElementPtr_V::iterator eiter;
@@ -273,9 +321,13 @@ void Element::PrintDocRightPane(std::string &_html, int _spacing, int &_index)
 
   stream << "<font style='font-weight:bold'>Description: </font>";
   if (!this->dataPtr->description.empty())
+  {
     stream << this->dataPtr->description << "<br>\n";
+  }
   else
+  {
     stream << "none<br>\n";
+  }
 
   stream << "<font style='font-weight:bold'>Required: </font>"
          << this->dataPtr->required << "&nbsp;&nbsp;&nbsp;\n";
@@ -289,7 +341,9 @@ void Element::PrintDocRightPane(std::string &_html, int _spacing, int &_index)
            << this->dataPtr->value->GetDefaultAsString() << '\n';
   }
   else
+  {
     stream << "n/a\n";
+  }
 
   stream << "</div>";
 
@@ -313,14 +367,18 @@ void Element::PrintDocRightPane(std::string &_html, int _spacing, int &_index)
       stream << "<div style='float:left; padding-left: 4px; width: 300px;'>\n";
 
       if (!(*aiter)->GetDescription().empty())
+      {
           stream << (*aiter)->GetDescription() << "<br>\n";
+      }
       else
+      {
           stream << "no description<br>\n";
+      }
 
       stream << "<font style='font-weight:bold'>Type: </font>"
              << (*aiter)->GetTypeName() << "&nbsp;&nbsp;&nbsp;"
-        << "<font style='font-weight:bold'>Default: </font>"
-        << (*aiter)->GetDefaultAsString() << "<br>";
+             << "<font style='font-weight:bold'>Default: </font>"
+             << (*aiter)->GetDefaultAsString() << "<br>";
       stream << "</div>\n";
 
       stream << "</div>\n";
@@ -335,7 +393,8 @@ void Element::PrintDocRightPane(std::string &_html, int _spacing, int &_index)
 }
 
 /////////////////////////////////////////////////
-void Element::PrintDocLeftPane(std::string &_html, int _spacing, int &_index)
+void Element::PrintDocLeftPane(std::string &_html, int _spacing,
+                               int &_index) const
 {
   std::ostringstream stream;
   ElementPtr_V::iterator eiter;
@@ -360,44 +419,60 @@ void Element::PrintDocLeftPane(std::string &_html, int _spacing, int &_index)
   _html += "</div>\n";
 }
 
-/////////////////////////////////////////////////
-void Element::PrintValues(std::string _prefix)
+void Element::PrintValuesImpl(const std::string &_prefix,
+                              std::ostringstream &_out) const
 {
-  std::cout << _prefix << "<" << this->dataPtr->name;
+  _out << _prefix << "<" << this->dataPtr->name;
 
-  Param_V::iterator aiter;
+  Param_V::const_iterator aiter;
   for (aiter = this->dataPtr->attributes.begin();
        aiter != this->dataPtr->attributes.end(); ++aiter)
   {
-    std::cout << " " << (*aiter)->GetKey() << "='"
-      << (*aiter)->GetAsString() << "'";
+    // Only print attribute values if they were set
+    // TODO(anyone): GetRequired is added here to support up-conversions where a
+    // new required attribute with a default value is added. We would have
+    // better separation of concerns if the conversion process set the required
+    // attributes with their default values.
+    if ((*aiter)->GetSet() || (*aiter)->GetRequired())
+    {
+      _out << " " << (*aiter)->GetKey() << "='"
+           << (*aiter)->GetAsString() << "'";
+    }
   }
 
   if (this->dataPtr->elements.size() > 0)
   {
-    std::cout << ">\n";
-    ElementPtr_V::iterator eiter;
+    _out << ">\n";
+    ElementPtr_V::const_iterator eiter;
     for (eiter = this->dataPtr->elements.begin();
-        eiter != this->dataPtr->elements.end(); ++eiter)
+         eiter != this->dataPtr->elements.end(); ++eiter)
     {
-      (*eiter)->PrintValues(_prefix + "  ");
+      (*eiter)->ToString(_prefix + "  ", _out);
     }
-    std::cout << _prefix << "</" << this->dataPtr->name << ">\n";
+    _out << _prefix << "</" << this->dataPtr->name << ">\n";
   }
   else
   {
     if (this->dataPtr->value)
     {
-      std::cout << ">" << this->dataPtr->value->GetAsString()
-        << "</" << this->dataPtr->name << ">\n";
+      _out << ">" << this->dataPtr->value->GetAsString()
+           << "</" << this->dataPtr->name << ">\n";
     }
     else
     {
-      std::cout << "/>\n";
+      _out << "/>\n";
     }
   }
 }
 
+/////////////////////////////////////////////////
+void Element::PrintValues(std::string _prefix) const
+{
+  std::ostringstream ss;
+  PrintValuesImpl(_prefix, ss);
+  std::cout << ss.str();
+}
+
 /////////////////////////////////////////////////
 std::string Element::ToString(const std::string &_prefix) const
 {
@@ -412,39 +487,7 @@ void Element::ToString(const std::string &_prefix,
 {
   if (this->dataPtr->includeFilename.empty())
   {
-    _out << _prefix << "<" << this->dataPtr->name;
-
-    Param_V::const_iterator aiter;
-    for (aiter = this->dataPtr->attributes.begin();
-        aiter != this->dataPtr->attributes.end(); ++aiter)
-    {
-      _out << " " << (*aiter)->GetKey() << "='"
-           << (*aiter)->GetAsString() << "'";
-    }
-
-    if (this->dataPtr->elements.size() > 0)
-    {
-      _out << ">\n";
-      ElementPtr_V::const_iterator eiter;
-      for (eiter = this->dataPtr->elements.begin();
-          eiter != this->dataPtr->elements.end(); ++eiter)
-      {
-        (*eiter)->ToString(_prefix + "  ", _out);
-      }
-      _out << _prefix << "</" << this->dataPtr->name << ">\n";
-    }
-    else
-    {
-      if (this->dataPtr->value)
-      {
-        _out << ">" << this->dataPtr->value->GetAsString()
-             << "</" << this->dataPtr->name << ">\n";
-      }
-      else
-      {
-        _out << "/>\n";
-      }
-    }
+    PrintValuesImpl(_prefix, _out);
   }
   else
   {
@@ -454,31 +497,35 @@ void Element::ToString(const std::string &_prefix,
 }
 
 /////////////////////////////////////////////////
-bool Element::HasAttribute(const std::string &_key)
+bool Element::HasAttribute(const std::string &_key) const
 {
-  return this->GetAttribute(_key) != NULL;
+  return this->GetAttribute(_key) != nullptr;
 }
 
 /////////////////////////////////////////////////
-bool Element::GetAttributeSet(const std::string &_key)
+bool Element::GetAttributeSet(const std::string &_key) const
 {
   bool result = false;
   ParamPtr p = this->GetAttribute(_key);
   if (p)
+  {
     result = p->GetSet();
+  }
 
   return result;
 }
 
 /////////////////////////////////////////////////
-ParamPtr Element::GetAttribute(const std::string &_key)
+ParamPtr Element::GetAttribute(const std::string &_key) const
 {
   Param_V::const_iterator iter;
   for (iter = this->dataPtr->attributes.begin();
       iter != this->dataPtr->attributes.end(); ++iter)
   {
     if ((*iter)->GetKey() == _key)
+    {
       return (*iter);
+    }
   }
   return ParamPtr();
 }
@@ -494,7 +541,9 @@ ParamPtr Element::GetAttribute(unsigned int _index) const
 {
   ParamPtr result;
   if (_index < this->dataPtr->attributes.size())
+  {
     result = this->dataPtr->attributes[_index];
+  }
 
   return result;
 }
@@ -510,7 +559,9 @@ ElementPtr Element::GetElementDescription(unsigned int _index) const
 {
   ElementPtr result;
   if (_index < this->dataPtr->elementDescriptions.size())
+  {
     result = this->dataPtr->elementDescriptions[_index];
+  }
   return result;
 }
 
@@ -522,14 +573,16 @@ ElementPtr Element::GetElementDescription(const std::string &_key) const
        iter != this->dataPtr->elementDescriptions.end(); ++iter)
   {
     if ((*iter)->GetName() == _key)
+    {
       return (*iter);
+    }
   }
 
   return ElementPtr();
 }
 
 /////////////////////////////////////////////////
-ParamPtr Element::GetValue()
+ParamPtr Element::GetValue() const
 {
   return this->dataPtr->value;
 }
@@ -537,15 +590,7 @@ ParamPtr Element::GetValue()
 /////////////////////////////////////////////////
 bool Element::HasElement(const std::string &_name) const
 {
-  ElementPtr_V::const_iterator iter;
-  for (iter = this->dataPtr->elements.begin();
-       iter != this->dataPtr->elements.end(); ++iter)
-  {
-    if ((*iter)->GetName() == _name)
-      return true;
-  }
-
-  return false;
+  return this->GetElementImpl(_name) != ElementPtr();
 }
 
 /////////////////////////////////////////////////
@@ -556,10 +601,11 @@ ElementPtr Element::GetElementImpl(const std::string &_name) const
        iter != this->dataPtr->elements.end(); ++iter)
   {
     if ((*iter)->GetName() == _name)
+    {
       return (*iter);
+    }
   }
 
-  // gzdbg << "Unable to find element [" << _name << "] return empty\n";
   return ElementPtr();
 }
 
@@ -567,9 +613,13 @@ ElementPtr Element::GetElementImpl(const std::string &_name) const
 ElementPtr Element::GetFirstElement() const
 {
   if (this->dataPtr->elements.empty())
+  {
     return ElementPtr();
+  }
   else
+  {
     return this->dataPtr->elements.front();
+  }
 }
 
 /////////////////////////////////////////////////
@@ -589,15 +639,21 @@ ElementPtr Element::GetNextElement(const std::string &_name) const
 
     ++iter;
     if (iter == parent->dataPtr->elements.end())
+    {
       return ElementPtr();
+    }
     else if (_name.empty())
+    {
       return *(iter);
+    }
     else
     {
       for (; iter != parent->dataPtr->elements.end(); ++iter)
       {
         if ((*iter)->GetName() == _name)
+        {
           return (*iter);
+        }
       }
     }
   }
@@ -605,13 +661,84 @@ ElementPtr Element::GetNextElement(const std::string &_name) const
   return ElementPtr();
 }
 
+/////////////////////////////////////////////////
+std::set<std::string> Element::GetElementTypeNames() const
+{
+  std::set<std::string> result;
+  auto elem = this->GetFirstElement();
+  while (elem)
+  {
+    std::string typeName = elem->GetName();
+    result.insert(typeName);
+    elem = elem->GetNextElement();
+  }
+  return result;
+}
+
+/////////////////////////////////////////////////
+bool Element::HasUniqueChildNames(const std::string &_type) const
+{
+  auto namedElementsCount = this->CountNamedElements(_type);
+  for (auto &iter : namedElementsCount)
+  {
+    if (iter.second > 1)
+    {
+      return false;
+    }
+  }
+  return true;
+}
+
+/////////////////////////////////////////////////
+std::map<std::string, std::size_t>
+Element::CountNamedElements(const std::string &_type) const
+{
+  std::map<std::string, std::size_t> result;
+
+  sdf::ElementPtr elem;
+  if (_type.empty())
+  {
+    elem = this->GetFirstElement();
+  }
+  else
+  {
+    elem = this->GetElementImpl(_type);
+  }
+
+  while (elem)
+  {
+    if (elem->HasAttribute("name"))
+    {
+      // Get("name") returns attribute value if it exists before checking
+      // for the value of a child element <name>, so it's safe to use
+      // here since we've checked HasAttribute("name").
+      std::string childNameAttributeValue = elem->Get<std::string>("name");
+      if (result.find(childNameAttributeValue) == result.end())
+      {
+        result[childNameAttributeValue] = 1;
+      }
+      else
+      {
+        ++result[childNameAttributeValue];
+      }
+    }
+
+    elem = elem->GetNextElement(_type);
+  }
+
+  return result;
+}
+
 /////////////////////////////////////////////////
 ElementPtr Element::GetElement(const std::string &_name)
 {
-  if (this->HasElement(_name))
-    return this->GetElementImpl(_name);
-  else
-    return this->AddElement(_name);
+  ElementPtr result = this->GetElementImpl(_name);
+  if (result == ElementPtr())
+  {
+    result = this->AddElement(_name);
+  }
+
+  return result;
 }
 
 /////////////////////////////////////////////////
@@ -621,21 +748,9 @@ void Element::InsertElement(ElementPtr _elem)
 }
 
 /////////////////////////////////////////////////
-bool Element::HasElementDescription(const std::string &_name)
+bool Element::HasElementDescription(const std::string &_name) const
 {
-  bool result = false;
-  ElementPtr_V::const_iterator iter;
-  for (iter = this->dataPtr->elementDescriptions.begin();
-       iter != this->dataPtr->elementDescriptions.end(); ++iter)
-  {
-    if ((*iter)->dataPtr->name == _name)
-    {
-      result = true;
-      break;
-    }
-  }
-
-  return result;
+  return this->GetElementDescription(_name) != ElementPtr();
 }
 
 /////////////////////////////////////////////////
@@ -684,6 +799,14 @@ ElementPtr Element::AddElement(const std::string &_name)
   return ElementPtr();
 }
 
+/////////////////////////////////////////////////
+void Element::Clear()
+{
+  this->ClearElements();
+  this->dataPtr->originalVersion.clear();
+  this->dataPtr->path.clear();
+}
+
 /////////////////////////////////////////////////
 void Element::ClearElements()
 {
@@ -712,7 +835,9 @@ void Element::Update()
   }
 
   if (this->dataPtr->value)
+  {
     this->dataPtr->value->Update();
+  }
 }
 
 /////////////////////////////////////////////////
@@ -722,7 +847,9 @@ void Element::Reset()
       iter != this->dataPtr->elements.end(); ++iter)
   {
     if (*iter)
+    {
       (*iter)->Reset();
+    }
     (*iter).reset();
   }
 
@@ -730,7 +857,9 @@ void Element::Reset()
       iter != this->dataPtr->elementDescriptions.end(); ++iter)
   {
     if (*iter)
+    {
       (*iter)->Reset();
+    }
     (*iter).reset();
   }
   this->dataPtr->elements.clear();
@@ -759,6 +888,30 @@ std::string Element::GetInclude() const
   return this->dataPtr->includeFilename;
 }
 
+/////////////////////////////////////////////////
+void Element::SetFilePath(const std::string &_path)
+{
+  this->dataPtr->path = _path;
+}
+
+/////////////////////////////////////////////////
+const std::string &Element::FilePath() const
+{
+  return this->dataPtr->path;
+}
+
+/////////////////////////////////////////////////
+void Element::SetOriginalVersion(const std::string &_version)
+{
+  this->dataPtr->originalVersion = _version;
+}
+
+/////////////////////////////////////////////////
+const std::string &Element::OriginalVersion() const
+{
+  return this->dataPtr->originalVersion;
+}
+
 /////////////////////////////////////////////////
 std::string Element::GetDescription() const
 {
@@ -792,7 +945,7 @@ void Element::RemoveFromParent()
 /////////////////////////////////////////////////
 void Element::RemoveChild(ElementPtr _child)
 {
-  SDF_ASSERT(_child, "Cannot remove a NULL child pointer");
+  SDF_ASSERT(_child, "Cannot remove a nullptr child pointer");
 
   ElementPtr_V::iterator iter;
   iter = std::find(this->dataPtr->elements.begin(),
@@ -806,15 +959,15 @@ void Element::RemoveChild(ElementPtr _child)
 }
 
 /////////////////////////////////////////////////
-boost::any Element::GetAny(const std::string &_key)
+std::any Element::GetAny(const std::string &_key) const
 {
-  boost::any result;
+  std::any result;
   if (_key.empty() && this->dataPtr->value)
   {
     if (!this->dataPtr->value->GetAny(result))
     {
       sdferr << "Couldn't get element [" << this->GetName()
-             << "] as boost::any\n";
+             << "] as std::any\n";
     }
   }
   else if (!_key.empty())
@@ -823,14 +976,30 @@ boost::any Element::GetAny(const std::string &_key)
     if (param)
     {
       if (!this->GetAttribute(_key)->GetAny(result))
-        sdferr << "Couldn't get attribute [" << _key << "] as boost::any\n";
+      {
+        sdferr << "Couldn't get attribute [" << _key << "] as std::any\n";
+      }
     }
-    else if (this->HasElement(_key))
-      result = this->GetElementImpl(_key)->GetAny();
-    else if (this->HasElementDescription(_key))
-      result = this->GetElementDescription(_key)->GetAny();
     else
-      sdferr << "Unable to find value for key [" << _key << "]\n";
+    {
+      ElementPtr tmp = this->GetElementImpl(_key);
+      if (tmp != ElementPtr())
+      {
+        result = tmp->GetAny();
+      }
+      else
+      {
+        tmp = this->GetElementDescription(_key);
+        if (tmp != ElementPtr())
+        {
+          result = tmp->GetAny();
+        }
+        else
+        {
+          sdferr << "Unable to find value for key [" << _key << "]\n";
+        }
+      }
+    }
   }
   return result;
 }
